/*
---------------------------------------------------------------------
CELL LEVEL ENCRYPTION
Code Source: "PROTECTING SQL SERVER DATA"
Author: John Magnabosco
Publisher: Simple Talk Publishing
Copyright September 2009
---------------------------------------------------------------------
Please note that it is highly discouraged to implement the provided sample code, 
either entirely or in part, on an instance of SQL Server that is actively utilized 
for production activity. All sample code is provided for illustrative purposes only 
and are provided as is without any warranties or guarantees of any kind, either expressed 
or implied. In no event shall the author or publisher be liable for any direct, indirect, 
incidental, special, exemplary, or consequential damages arising in any way out of the use 
of the provided sample code.
---------------------------------------------------------------------
*/

/* Listing 5-1: Denying access to the base tables */
USE HomeLending;
GO

DENY ALTER,CONTROL,REFERENCES,DELETE,INSERT,UPDATE,SELECT
    ON dbo.Borrower_Identification 
    TO Sensitive_high, Sensitive_medium, Sensitive_low
GO

/* Listing 5-2: Confirming the existence of the Service Master Key */
USE master;
GO

SELECT 
    * 
FROM 
    sys.symmetric_keys
WHERE 
    name = '##MS_ServiceMasterKey##';
GO

/* Listing 5-3: Creating the Database Master Key */
USE HomeLending;
GO

CREATE MASTER KEY 
    ENCRYPTION BY PASSWORD = 'MyStr0ngP@ssw0rd2009';
GO

/* Listing 5-4: Confirming protection of the database master key by the service master key */
USE HomeLending;
GO

SELECT 
    b.name,
    a.crypt_type_desc 
FROM 
    sys.key_encryptions a
    INNER JOIN sys.symmetric_keys b
        ON a.key_id = b.symmetric_key_id
WHERE 
    b.name = '##MS_DatabaseMasterKey##';
GO

/* Listing 5-5: Creating the MyHighCert self-signed certificate */
USE HomeLending;
GO

CREATE CERTIFICATE MyHighCert 
    WITH SUBJECT = 'Cert used for sensitive class of high';
GO

/* Listing 5-6: Creating the HighSymKey1 symmetric key */
USE HomeLending;
GO

-- A 128 bit Symmetric key (strong)
CREATE SYMMETRIC KEY HighSymKey1
    WITH ALGORITHM = AES_128
    ENCRYPTION BY CERTIFICATE MyHighCert;
GO

/* Listing 5-7: Granting the VIEW DEFINITION permission to the Sensitive_high database role */
USE HomeLending;
GO

-- Used By Sensitive_high
GRANT VIEW DEFINITION ON SYMMETRIC KEY::HighSymKey1
    TO Sensitive_high;
GO

/* Listing 5-8: Validating the access to key hierarchy */
USE HomeLending;
GO

-- execute as a user who is a member of Sensitive_high role
EXECUTE AS USER = 'WOLFBA'; 
GO
SELECT * FROM sys.symmetric_keys;
GO
REVERT;
GO

-- execute as a user who is a member of Sensitive_medium role
EXECUTE AS USER = 'KELLEYWB'; 
GO
SELECT * FROM sys.symmetric_keys;
GO
REVERT;
GO

-- execute as a user who is a member of Sensitive_low role
EXECUTE AS USER = 'JONESBF'; 
GO
SELECT * FROM sys.symmetric_keys;
GO
REVERT;
GO

/* Listing 5-9: Adding a column to store varbinary data */
USE HomeLending;
GO

ALTER TABLE dbo.Borrower_Identification ADD
    Identification_Value_E varbinary(MAX) NULL
GO

/* Listing 5-10: Encrypting the data for the Identification_Value_E column */
USE HomeLending;
GO

-- Opens the symmetric key for use
OPEN SYMMETRIC KEY HighSymKey1
    DECRYPTION BY CERTIFICATE MyHighCert;
GO

-- Performs the update of the record
UPDATE dbo.Borrower_Identification
    SET Identification_Value_E = 
        EncryptByKey(
            Key_GUID('HighSymKey1'),
            Identification_Value,
            1,
            CONVERT(nvarchar(128),Borrower_ID)
        )
FROM
    dbo.Borrower_Identification;
GO

-- Closes the symmetric key    
CLOSE SYMMETRIC KEY HighSymKey1;
GO

/* Listing 5-11: Dropping the plain text Identification_Value column */
USE HomeLending;
GO

ALTER TABLE dbo.Borrower_Identification
    DROP COLUMN Identification_Value;
GO


/* Listing 5-12: Documenting the encrypted column */
USE HomeLending;
GO

EXEC sp_addextendedproperty 
   @name='Sensitivity_Class',
   @value='High',
   @level0type='SCHEMA',
   @level0name='dbo',
   @level1type='TABLE',
   @level1name='Borrower_Identification',
   @level2type='COLUMN',
   @level2name='Identification_Value_E';
GO

/* Listing 5-13: Creating the vwBorrower_Identification view */
USE HomeLending;
GO

CREATE VIEW dbo.vwBorrower_Identification
AS

SELECT
    Borrower_Identification_ID,
    Borrower_ID,
    Identification_Type_ID,
    CONVERT(varchar(250),
        COALESCE(
            DecryptByKeyAutoCert (
                CERT_ID('MyHighCert'), 
                NULL,
                Identification_Value_E,
                1,
                CONVERT(nvarchar(128), Borrower_ID)
            ),
            '<SECURED VALUE>'
        )
    ) AS Identification_Value
FROM
    dbo.Borrower_Identification;
GO

/* Listing 5-14: Granting permission to access the view */
USE HomeLending;
GO

GRANT SELECT ON dbo.vwBorrower_Identification 
    TO Sensitive_high, Sensitive_medium;
GO

/* Listing 5-15: Verification of permissions to vwBorrower_Identification  */
USE HomeLending;
GO

-- execute as a user who is a member of Sensitive_high role
EXECUTE AS USER = 'WOLFBA'; 
GO
SELECT * FROM dbo.vwBorrower_Identification;
GO
REVERT;
GO

-- execute as a user who is a member of Sensitive_medium role
EXECUTE AS USER = 'KELLEYWB'; 
GO
SELECT * FROM dbo.vwBorrower_Identification;
GO
REVERT;
GO

-- execute as a user who is a member of Sensitive_low role
EXECUTE AS USER = 'JONESBF'; 
GO
SELECT * FROM dbo.vwBorrower_Identification;
GO
REVERT;
GO

/* Listing 5-16: The stored procured through which authorized users can update sensitive data */
USE HomeLending;
GO

CREATE PROCEDURE dbo.Update_Borrower_Identification 
    @Borrower_Identification_ID bigint,
    @Borrower_ID bigint,
    @Identification_Value varchar(250)
AS
BEGIN TRY

    -- Opens the symmetric key for use
    OPEN SYMMETRIC KEY HighSymKey1
        DECRYPTION BY CERTIFICATE MyHighCert;

    -- Performs the update of the record
    UPDATE dbo.Borrower_Identification
        SET Identification_Value_E = 
            EncryptByKey(
                Key_GUID('HighSymKey1'),
                @Identification_Value,
                1,
                CONVERT(nvarchar(128),@Borrower_ID)
            )
    WHERE 
        Borrower_Identification_ID = 
            @Borrower_Identification_ID;
            
END TRY
BEGIN CATCH
    -- Returns the error information
    SELECT
        ERROR_NUMBER() AS ErrorNumber,
        ERROR_SEVERITY() AS ErrorSeverity,
        ERROR_STATE() AS ErrorState,
        ERROR_MESSAGE() AS ErrorMessage;
END CATCH

-- Closes the symmetric key if open
IF(EXISTS(
            SELECT 
                * 
            FROM 
                sys.openkeys 
            WHERE 
                key_guid = Key_GUID('HighSymKey1')
          )
    )
    BEGIN
        CLOSE SYMMETRIC KEY HighSymKey1;
    END
GO

/* Listing 5-17: The stored procured through which authorized users can insert sensitive data */
USE HomeLending;
GO

CREATE PROCEDURE dbo.Insert_Borrower_Identification 
    @Borrower_ID bigint,
    @Identification_Type_ID int,
    @Identification_Value varchar(250)
AS
BEGIN TRY

    -- Opens the symmetric key for use
    OPEN SYMMETRIC KEY HighSymKey1
        DECRYPTION BY CERTIFICATE MyHighCert;

    -- Performs the update of the record
    INSERT INTO dbo.Borrower_Identification
        (
            Borrower_ID,
            Identification_Type_ID,
            Identification_Value_E
        )
        VALUES
        (
            @Borrower_ID,
            @Identification_Type_ID,
            EncryptByKey(
                Key_GUID('HighSymKey1'),
                @Identification_Value,
                1,
                CONVERT(nvarchar(128),@Borrower_ID)
            )
        );
    
    -- Captures the new Borrower_Identifcation_ID value
    SELECT @@IDENTITY;
            
END TRY
BEGIN CATCH
    -- Returns the error information
    SELECT
        ERROR_NUMBER() AS ErrorNumber,
        ERROR_SEVERITY() AS ErrorSeverity,
        ERROR_STATE() AS ErrorState,
        ERROR_MESSAGE() AS ErrorMessage;
END CATCH

-- Closes the symmetric key if open
IF(EXISTS(
            SELECT 
                * 
            FROM 
                sys.openkeys 
            WHERE 
                key_guid = Key_GUID('HighSymKey1')
          )
    )
    BEGIN
        CLOSE SYMMETRIC KEY HighSymKey1;
    END
GO

/* Listing 5-18: Granting, to the sensitive_high role, permission to execute the two stored procedures  */
USE HomeLending;
GO

-- Grant Execute Permissions to Sensitive_high database role
GRANT EXECUTE ON dbo.Update_Borrower_Identification 
    TO Sensitive_high;
GO

-- Grant Execute Permissions to Sensitive_high database role
GRANT EXECUTE ON dbo.Insert_Borrower_Identification 
    TO Sensitive_high;
GO






