/*
---------------------------------------------------------------------
OBFUSCATION 
Code Source: "PROTECTING SQL SERVER DATA"
Author: John Magnabosco
Publisher: Simple Talk Publishing
Copyright September 2009
---------------------------------------------------------------------
Please note that it is highly discouraged to implement the provided sample code, 
either entirely or in part, on an instance of SQL Server that is actively utilized 
for production activity. All sample code is provided for illustrative purposes only 
and are provided as is without any warranties or guarantees of any kind, either expressed 
or implied. In no event shall the author or publisher be liable for any direct, indirect, 
incidental, special, exemplary, or consequential damages arising in any way out of the use 
of the provided sample code.
---------------------------------------------------------------------
*/

/* Listing 8-1: Generating random numbers using RAND */
Use HomeLending;
GO

-- Used to reference RAND with in a function
CREATE VIEW dbo.vwRandom
AS
SELECT RAND() as RandomValue;
GO

-- Grant permissions to view
GRANT SELECT ON dbo.vwRandom
	TO Sensitive_high, Sensitive_medium, Sensitive_low;
GO

/* Listing 8-2: Testing the View */
DECLARE @Rand float;
DECLARE @MinVal int;
DECLARE @MaxVal int;
SET @MinVal = 1;
SET @MaxVal = 100;

SELECT
     @Rand = ((@MinVal + 1) - @MaxVal) * RandomValue + @MaxVal
FROM 
     dbo.vwRandom;
GO

/* Listing 8-3: The character scrambling UDF */
Use HomeLending;
GO

-- Create user defined function
CREATE FUNCTION Character_Scramble 
(
    @OrigVal varchar(max)
)
RETURNS varchar(max)
WITH ENCRYPTION
AS
BEGIN

-- Variables used
DECLARE @NewVal varchar(max);
DECLARE @OrigLen int;
DECLARE @CurrLen int;
DECLARE @LoopCt int;
DECLARE @Rand int;

-- Set variable default values
SET @NewVal = '';
SET @OrigLen = DATALENGTH(@OrigVal);
SET @CurrLen = @OrigLen;
SET @LoopCt = 1;

-- Loop through the characters passed
WHILE @LoopCt <= @OrigLen
    BEGIN
        -- Current length of possible characters
        SET @CurrLen = DATALENGTH(@OrigVal);
    
        -- Random position of character to use
        SELECT
            @Rand = Convert(int,(((1) - @CurrLen) *  
                               RandomValue + @CurrLen))
        FROM
            dbo.vwRandom;
            
        -- Assembles the value to be returned 
        SET @NewVal = @NewVal + 
                             SUBSTRING(@OrigVal,@Rand,1);
        
        -- Removes the character from available options
        SET @OrigVal = 
                 Replace(@OrigVal,SUBSTRING(@OrigVal,@Rand,1),'');

        -- Advance the loop
        SET @LoopCt = @LoopCt + 1;
    END
    -- Returns new value
    Return LOWER(@NewVal);
END
GO

-- Grant permissions to user defined function
GRANT EXECUTE ON dbo.Character_Scramble
    TO Sensitive_high, Sensitive_medium, Sensitive_low;
GO

/* Listing 8-4: The Character_Mask UDF */
Use HomeLending;
GO

-- Create user defined function
CREATE FUNCTION Character_Mask 
(
    @OrigVal varchar(max),
    @InPlain int,
    @MaskChar char(1)
)
RETURNS varchar(max)
WITH ENCRYPTION
AS
BEGIN

    -- Variables used
    DECLARE @PlainVal varchar(max);
    DECLARE @MaskVal varchar(max);
    DECLARE @MaskLen int;
    
    -- Captures the portion of @OrigVal that remains in plain text
    SET @PlainVal = RIGHT(@OrigVal,@InPlain);
    -- Defines the length of the repeating value for the mask
    SET @MaskLen = (DATALENGTH(@OrigVal) - @InPlain);
    -- Captures the mask value
    SET @MaskVal = REPLICATE(@MaskChar, @MaskLen);
    -- Returns the masked value    
    Return @MaskVal + @PlainVal;

END
GO

-- Grant permissions to user defined function
GRANT EXECUTE ON dbo.Character_Mask
    TO Sensitive_high, Sensitive_medium, Sensitive_low;
GO

/* Listing 8-5: The vwLoanBorrowers View */
Use HomeLending;
GO

CREATE VIEW dbo.vwLoanBorrowers
AS
SELECT
    dbo.Character_Mask(ln.Lender_Case_Number,4,'X') 
        AS Lender_Case_Number,
    dbo.Character_Scramble(bn.Borrower_FName + ' ' 
        + bn.Borrower_LName)
        AS Borrower_Name
FROM
    dbo.Loan ln
    INNER JOIN dbo.Loan_Borrowers lb
        ON ln.Loan_ID = lb.Loan_ID
        AND lb.Borrower_Type_ID = 1 -- Primary Borrowers Only
    INNER JOIN dbo.Borrower_Name bn
        ON lb.Borrower_ID = bn.Borrower_ID;
GO
        
-- Grant permissions to view
GRANT SELECT ON dbo.vwLoanBorrowers
    TO Sensitive_high, Sensitive_medium;
GO

/* Listing 8-6: The Numeric_Variance UDF */
USE HomeLending;
GO

-- Create user defined function
CREATE FUNCTION Numeric_Variance 
(
    @OrigVal float,
    @VarPercent numeric(5,2)
)
RETURNS float
WITH ENCRYPTION
AS
BEGIN
    -- Variable used
    DECLARE @Rand int;
    
    -- Random position of character to use
    SELECT
        @Rand = Convert(int,((((0-@VarPercent)+1) - 
               @VarPercent) * RandomValue + @VarPercent))
    FROM
        dbo.vwRandom;

    RETURN @OrigVal + CONVERT(INT,((@OrigVal*@Rand)/100));
END
GO

-- Grant permissions to user defined function
GRANT EXECUTE ON dbo.Numeric_Variance
    TO Sensitive_high, Sensitive_medium, Sensitive_low;
GO

/* Listing 8-7: Updating a development database to use numeric variance */
USE HomeLending;
GO
    
-- Variables used
DECLARE @Variance numeric(5,2)
-- Set variance to 10%
SET @Variance = 10

UPDATE dbo.Loan_Term
    SET Loan_Amount =
            dbo.Numeric_Variance(Loan_Amount,@Variance)
FROM
    dbo.Loan_Term;
GO

/* Listing 8-8: Nulling a database column */
USE HomeLending;
GO
    
UPDATE dbo.Borrower_Identification
    SET Identification_Value = NULL
FROM
    dbo.Borrower_Identification;
GO

/* Listing 8-9: Permanently truncating the Identification_Value column */
USE HomeLending;
GO
    
UPDATE dbo.Borrower_Identification
    SET Identification_Value = 
              LEFT(Identification_Value,4)
FROM
    dbo.Borrower_Identification;
GO

/* Listing 8-10: Returning a truncated value */
USE HomeLending;
GO
    
SELECT
    LEFT(Identification_Value,4) AS Identification_Value
FROM
    dbo.Borrower_Identification;
GO









