/*
---------------------------------------------------------------------
ONE WAY ENCRYPTION 
Code Source: "PROTECTING SQL SERVER DATA"
Author: John Magnabosco
Publisher: Simple Talk Publishing
Copyright September 2009
---------------------------------------------------------------------
Please note that it is highly discouraged to implement the provided sample code, 
either entirely or in part, on an instance of SQL Server that is actively utilized 
for production activity. All sample code is provided for illustrative purposes only 
and are provided as is without any warranties or guarantees of any kind, either expressed 
or implied. In no event shall the author or publisher be liable for any direct, indirect, 
incidental, special, exemplary, or consequential damages arising in any way out of the use 
of the provided sample code.
---------------------------------------------------------------------
*/

/* Listing 7-1: The GetHashSalt UDF */
Use HomeLending;
GO

CREATE FUNCTION GetHashSalt
(
	@Type varchar(3)
)
RETURNS varchar(7)
WITH ENCRYPTION
AS
BEGIN

DECLARE @Rtn varchar(7)

IF @Type = 'L01' SET @Rtn = 'HYz5#45';
IF @Type = 'L02' SET @Rtn = 'Ku&7723';
IF @Type = 'L03' SET @Rtn = 'PW2%230';
IF @Type = 'L04' SET @Rtn = 'T^542Xc';
IF @Type = 'L05' SET @Rtn = '89*we@4';
IF @Type = 'L06' SET @Rtn = '098&tsS';

RETURN @Rtn;

END
GO

/* Listing 7-2: Granting permissions to the GetHashSalt UDF */
Use HomeLending;
GO

GRANT EXECUTE ON dbo.GetHashSalt
	TO Sensitive_high, Sensitive_medium;
GO

/* Listing 7-3: Creating the hash column */
USE HomeLending;
GO

ALTER TABLE dbo.Borrower_Identification 
	ADD Identification_Value_H varbinary(MAX) NULL;
GO

/* Listing 7-4: Documenting the encrypted column as "high" sensitivity */
USE HomeLending;
GO

EXEC sp_addextendedproperty 
   @name='Sensitivity_Class',
   @value='High',
   @level0type='SCHEMA',
   @level0name='dbo',
   @level1type='TABLE',
   @level1name='Borrower_Identification',
   @level2type='COLUMN',
   @level2name='Identification_Value_H';
GO

/* Listing 7-5: Creating and documenting the secondary hash column */
USE HomeLending;
GO

ALTER TABLE dbo.Borrower_Identification 
	ADD Identification_Value_HT varbinary(MAX) NULL;
GO

EXEC sp_addextendedproperty 
   @name='Sensitivity_Class',
   @value='Medium',
   @level0type='SCHEMA',
   @level0name='dbo',
   @level1type='TABLE',
   @level1name='Borrower_Identification',
   @level2type='COLUMN',
   @level2name='Identification_Value_HT';
GO

/* Listing 7-6: Salting and hashing the Identification_Value column */
USE HomeLending;
GO

DECLARE @Salt varchar(7)
SET @Salt = dbo.GetHashSalt('L01');

UPDATE dbo.Borrower_Identification
   SET Identification_Value_H = 
          HASHBYTES('SHA1', @Salt + 
              Identification_Value), 
       Identification_Value_HT = 
          HASHBYTES('SHA1', @Salt + 
              RIGHT(Identification_Value,4));	 
GO

/* Listing 7-7: Testing the one-way encryption */
USE HomeLending;
GO

DECLARE @Salt varchar(7)
SET @Salt = dbo.GetHashSalt('L01');

-- Returns search by full SSN
SELECT
   Identification_Value
FROM 
   dbo.Borrower_Identification
WHERE
   Identification_Value_H = 
        HASHBYTES('SHA1', @Salt + '555-20-7151');

-- Returns search by last four digits of SSN	
SELECT
   Identification_Value
FROM 
   dbo.Borrower_Identification
WHERE
   Identification_Value_HT = 
        HASHBYTES('SHA1', @Salt + '7151');
GO

/* Listing 7-8: Dropping the plain-text column */
USE HomeLending;
GO

ALTER TABLE dbo.Borrower_Identification
	DROP COLUMN Identification_Value;
GO 

/* Listing 7-9: Denying access to the base tables */
USE HomeLending;
GO

DENY ALTER,CONTROL,REFERENCES,DELETE,INSERT,UPDATE,SELECT
	ON dbo.Borrower_Identification 
	TO Sensitive_high, Sensitive_medium, Sensitive_low
GO

/* Listing 7-10: Recreating the vwBorrower_Identification view */
USE HomeLending;
GO

-- Create the view
CREATE VIEW dbo.vwBorrower_Identification
AS

SELECT
	Borrower_Identification_ID,
	Borrower_ID,
	Identification_Type_ID,
       Identification_Value_HT
FROM
	dbo.Borrower_Identification;
GO

-- Grant Select Permissions
GRANT SELECT ON dbo.vwBorrower_Identification 
	TO Sensitive_high, Sensitive_medium;
GO

/* Listing 7-11: Creating the Update_Borrower_Identification stored procedure */
USE HomeLending;
GO

CREATE PROCEDURE dbo.Update_Borrower_Identification
	@Borrower_Identification_ID bigint,
	@Identification_Value varchar(250)
AS
BEGIN TRY

	-- Get Salt
	DECLARE @Salt varchar(7)
	SET @Salt = dbo.GetHashSalt('L01');

	-- Performs the update of the record
	UPDATE dbo.Borrower_Identification
	   SET Identification_Value_H = 
	           HashBytes('SHA1',@Salt + 
                      @Identification_Value),
               Identification_Value_HT = 
		   HashBytes('SHA1',@Salt +                 
                      Right(@Identification_Value,4))
	WHERE 
		Borrower_Identification_ID = 
			@Borrower_Identification_ID;
			
END TRY
BEGIN CATCH
	-- Returns the error information
    SELECT
        ERROR_NUMBER() AS ErrorNumber,
        ERROR_SEVERITY() AS ErrorSeverity,
        ERROR_STATE() AS ErrorState,
        ERROR_MESSAGE() AS ErrorMessage;
END CATCH
GO

/* Listing 7-12: Creating the Insert_Borrower_Identification stored procedure */
USE HomeLending;
GO

CREATE PROCEDURE dbo.Insert_Borrower_Identification 
	@Borrower_ID bigint,
	@Identification_Type_ID int,
	@Identification_Value varchar(250)
AS
BEGIN TRY

	-- Get Salt
	DECLARE @Salt varchar(7)
	SET @Salt = dbo.GetHashSalt('L01');

	-- Performs the update of the record
	INSERT INTO dbo.Borrower_Identification
		(
			Borrower_ID,
			Identification_Type_ID,
			Identification_Value_H,
			Identification_Value_HT
		)
		VALUES
		(
			@Borrower_ID,
			@Identification_Type_ID,
			HashBytes('SHA1',@Salt +
                          @Identification_Value),
			HashBytes('SHA1',@Salt + 
                          Right(@Identification_Value,4))
		);
	
	-- Captures the new Borrower_Identifcation_ID value
	SELECT @@IDENTITY;
			
END TRY
BEGIN CATCH
	-- Returns the error information
    SELECT
        ERROR_NUMBER() AS ErrorNumber,
        ERROR_SEVERITY() AS ErrorSeverity,
        ERROR_STATE() AS ErrorState,
        ERROR_MESSAGE() AS ErrorMessage;
END CATCH
GO

/* Listing 7-13: Creating the Search_Borrower_Identification stored procedure */
USE HomeLending;
GO

CREATE PROCEDURE dbo.Search_Borrower_Identification 
	@Identification_Value varchar(4)
AS
BEGIN TRY

	-- Get Salt
	DECLARE @Salt varchar(7)
	SET @Salt = dbo.GetHashSalt('L01');

	-- Return Search Results
	SELECT
		bn.Borrower_LName,
		bn.Borrower_FName,
		bn.Borrower_MName,
		bn.Borrower_Suffix
	FROM
		dbo.Borrower_Identification bi
		INNER JOIN dbo.Borrower_Name bn
			ON bi.Borrower_ID = bn.Borrower_ID
	WHERE
		bi.Identification_Value_HT = 
			HashBytes('SHA1',@Salt + 
                           @Identification_Value);
			
END TRY
BEGIN CATCH
	-- Returns the error information
    SELECT
        ERROR_NUMBER() AS ErrorNumber,
        ERROR_SEVERITY() AS ErrorSeverity,
        ERROR_STATE() AS ErrorState,
        ERROR_MESSAGE() AS ErrorMessage;
END CATCH
GO

/* Listing 7-14: Setting permissions to the stored procedures */
USE HomeLending;
GO

-- Grant Execute Permissions to Sensitive_high database role
GRANT EXECUTE ON dbo.Update_Borrower_Identification 
	TO Sensitive_high;
GO

-- Grant Execute Permissions to Sensitive_high database role
GRANT EXECUTE ON dbo.Insert_Borrower_Identification
	TO Sensitive_high;
GO

-- Grant Execute Permissions to Sensitive_high 
-- and Sensitive_medium database roles
GRANT EXECUTE ON dbo.Search_Borrower_Identification 
	TO Sensitive_high, Sensitive_medium;
GO

/* Listing 7-15: Verifying permissions */
USE HomeLending;
GO

-- execute as a user who is a member of Sensitive_high role
EXECUTE AS USER = 'WOLFBA'; 
GO
Exec dbo.Search_Borrower_Identification '0143';
GO
REVERT;
GO

-- execute as a user who is a member of Sensitive_medium role
EXECUTE AS USER = 'KELLEYWB'; 
GO
Exec dbo.Search_Borrower_Identification '0143';
GO
REVERT;
GO

-- execute as a user who is a member of Sensitive_low role
EXECUTE AS USER = 'JONESBF'; 
GO
Exec dbo.Search_Borrower_Identification '0143';
GO
REVERT;
GO


